package com.berbix.berbix_flutter

import android.app.Activity
import android.content.Intent
import android.os.Build
import androidx.annotation.NonNull
import com.berbix.berbixverify.*
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener


class BerbixFlutterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware, ActivityResultListener {
	/// The MethodChannel that will the communication between Flutter and native Android
	///
	/// This local reference serves to register the plugin with the Flutter Engine and unregister it
	/// when the Flutter Engine is detached from the Activity
	private lateinit var channel : MethodChannel

	private var berbixSDK: BerbixSDK? = null
	private var config: BerbixConfiguration? = null
	private var currentActivity: Activity? = null
	private var currentBinding: ActivityPluginBinding? = null
	private var currentFlowResult: Result? = null

	override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
		setUpSDK(flutterPluginBinding)
	}

	override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
		channel.setMethodCallHandler(null)
		updateAttached(null)
	}

	override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
		when (call.method) {
			"getPlatformVersion" -> {
				result.success("Android ${Build.VERSION.RELEASE}")
			}
			"setClientToken" -> {
				setClientToken(call.arguments<String>(), result)
			}
			"startFlow" -> {
				startFlow(result)
			}
			else -> {
				result.notImplemented()
			}
		}
	}

	//
	// 	Flutter Activity Aware
	//
	override fun onAttachedToActivity(binding: ActivityPluginBinding) {
		updateAttached(binding)
	}

	override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
		updateAttached(binding)
	}

	override fun onDetachedFromActivityForConfigChanges() {
		updateAttached(null)
	}

	override fun onDetachedFromActivity() {
		updateAttached(null)
	}

	//
	//	Activity Result Listener
	//
	override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
		if (requestCode == BerbixConstants.REQUEST_CODE_BERBIX_FLOW) {
			val outerParent: BerbixFlutterPlugin = this
			BerbixResultStatus.handleResult(resultCode, data, object : BerbixResultListener {

				override fun onComplete() {
					print("flow completed")
					outerParent.currentFlowResult?.success(true)

				}

				override fun onFailure(status: BerbixErrorReason, reason: String?) {
					print("Failure reason: $reason")
					outerParent.currentFlowResult?.error(status.name, reason, null)
					when (status) {
						BerbixErrorReason.USER_EXIT -> print("user exited flow")
						BerbixErrorReason.NO_CAMERA_PERMISSIONS -> print("no camera permission")
						BerbixErrorReason.UNABLE_TO_LAUNCH_CAMERA -> print("could not launch camera")
						BerbixErrorReason.BERBIX_ERROR -> print("Berbix error occurred")
						BerbixErrorReason.UNKNOWN_ERROR -> print("unknown error received")
						else -> print("unknown error received")
					}
				}
			})
		}

		return true
	}

	//
	// 	Private Helpers
	//
	private fun setUpSDK(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
		berbixSDK = BerbixSDK()

		// This channel name is shared between dart, android and iOS. Don't change this unless it's
		// changed in the remaining platform files.
		channel = MethodChannel(binding.binaryMessenger, "flutter.berbix.com/sdk")
		channel.setMethodCallHandler(this)
	}

	private fun updateAttached(newBinding: ActivityPluginBinding?) {
		if (newBinding != null) {
			currentActivity = newBinding.activity
			currentBinding = newBinding
			currentBinding?.addActivityResultListener(this)
		} else {
			currentActivity = null
			currentBinding?.removeActivityResultListener(this)
			currentBinding = null
			currentFlowResult = null
		}
	}

	private fun setClientToken(clientToken: String, result: Result) {
		print("Updating Android client identifier to $clientToken")
		config = BerbixConfigurationBuilder()
				.setClientToken(clientToken)
				.build()
		result.success(true)
	}

	private fun startFlow(result: Result) {
		if (this.currentActivity != null && this.config != null) {
			print("Starting flow with activity: ${this.currentActivity}")
			this.currentFlowResult = result
			berbixSDK?.startFlow(this.currentActivity!!, this.config!!)
		} else {
			print("Failed to start flow; unable to determine current activity!")
			result.error("start-flow", "Failed to initiate flow from top level activity", null)
		}
	}
}